#include <tiams.h>
#include "Complex.h"
#include "CZT.h"
#include "FFT.h"
#include "Matrix.h"
#include "Real.h"
#include "ROM.h"
#include "Stack.h"

/* chirp Z transform (using convolution) */
void czt(void)
{	
	Access_AMS_Global_Variables; 

	EStackIndex argptr = top_estack;
   	volatile HANDLE inHdl, outHdl;
	double *inPtr;
	Complex *outPtr;
    unsigned short n, m; 
	Complex w, a;
	
	// check number of parameters
	CheckNumArgs (4);
		    
	TRY	
		// read samples into array
		n = GetFloatListArg (argptr, &inHdl);
		
		// list is empty
		if (n < 1) {
			ER_throwVar (ER_DOMAIN);
		}	
		
		argptr = next_expression_index (argptr);
		
		// get output length
		m = GetValue (argptr, 1, MAX_LST_LEN); 
		
		argptr = next_expression_index (argptr);

		// get ratio between points
		w = estack_number_to_Complex (argptr);
		
		argptr = next_expression_index (argptr);

		// get starting point
		a = estack_number_to_Complex (argptr);
		
		outHdl = HeapAllocThrow (m * sizeof (Complex));
						
		inPtr = HeapDeref (inHdl);
		outPtr = HeapDeref (outHdl);
	
		// perform the Chirp Z Transform
		CZT(inPtr,	// input signal
			n,			// length of input 
			outPtr,		// output signal
			m,          // length of output
			w,          // ratio between points
			a);         // starting point
		
		// push result
		push_complex_float_list ((Complex *)outPtr, m);
	FINALLY
		// avoid address error when arg is empty list
		HeapFreeIndir ((HANDLE *)&inHdl); 
		HeapFreeIndir ((HANDLE *)&outHdl);
	ENDFINAL
}

/* chirp Z Transform (by concoluation 
input signal
length of input signal
output signal
length of output signal
ratio between points
starting point on spiral contour
FFT: CZT(x, length (x), length (x), exp (-i * 2 * pi / m), 1)
loosely based on Grant D. Martin: Chirp Z-Transform Spectral Zoom Optimization with MATLAB (equation 1) */
void CZT(double x[],
	unsigned short n,
	Complex y[],
	unsigned short m, 
	Complex w,
	Complex a)
{
	unsigned short i, j;
	Complex s, t;
	
	// loop over output samples
	for (i = 0; i < m; i++) {
		s.real = 0.;
		s.imag = 0.;
		
		// this does not depend on j and can be calculated before the loop
		t = CMul (a, CPow (w, Double2Comp (-i))); 
		
		// loop over input samples
		for (j = 0; j < n; j++) {
			s = CAdd (s, CMul (Double2Comp (x[j]), CPow (t, Double2Comp (-j)))); 
		}
		
		y[i] = s;
	}
}
  