/*
		Estack interaction
		
    Copyright (C) 2015  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// http://stackoverflow.com/questions/8767166/passing-a-2d-array-to-a-c-function

#include <tiams.h>
#include "Complex.h"
#include "Memory.h"
#include "Stack.h"
#include "Wrapper.h"

// A R G U M E M T   C H E C K I N G //////////////////////////////////////////

/*********************************************************/
/* Title: remaining_element_count */
/* Description: Get number of expressions in tail. */
/* Input: estack index i */
/* i indexes a tail of expressions terminated */
/* by END_TAG. */
/* Output: unsigned number of expressions in the tail */
/*********************************************************/
unsigned short remaining_element_count (EStackIndex i) {
	unsigned short ans = 0u;   
	
	while (ESTACK (i) != END_TAG) {
		++ans;
		i = next_expression_index (i);
	}
	
	return ans;
}

/* check if number of arguments is correct */
unsigned char CheckNumArgs(unsigned char expArgs)
{
	Access_AMS_Global_Variables;
	unsigned char args = remaining_element_count (top_estack);
	
	// check number of arguments
	if (args < expArgs) {
		ER_throwVar (TOO_FEW_ARGS_ERROR);
	} else if (args > expArgs) {
		ER_throwVar (TOO_MANY_ARGS_ERROR);
	}	
	
	return args;
}

/* check if number of arguments is correct (interval) */
unsigned char CheckNumArgsInt(unsigned char minExpArgs, 
	unsigned char maxExpArgs)
{
	Access_AMS_Global_Variables;
	char args = remaining_element_count (top_estack);
	
	// check number of arguments
	if (args < minExpArgs) {
		ER_throwVar (TOO_FEW_ARGS_ERROR);
	} else if (args > maxExpArgs) {
		ER_throwVar (TOO_MANY_ARGS_ERROR);
	}	
	
	return args;
}

/* get type of argument with number (0-based index) */
/*enum Tags GetArgNumType(short argNum)
{
	return GetArgType (EX_getArg (argNum));
}*/

/* check if char argument is contained in list
returns the value of the argument at position argPos
??? this function should also be implemented for strings */
/*const char *CheckArgCharVal(unsigned short argPos, 
	char *allowedVal)
{
	// argument at position does not exist
	if (argPos > ArgCount ()) {
		ER_throw (ER_TOO_FEW_ARGS);		
	}
	
	// get pointer to argument at position
	CESI argptr = EX_getArg (argPos);
	
	// argument has wrong type
	if (GetArgType (argptr) != STR_TAG) {
		ER_throw (ER_ARG_MUST_BE_STRING);		
	}
			
	// get value of argument at position
	const char *arg = GetStrnArg (argptr);
				
	// check if val is not empty string and contained in string
	if (strstr (allowedVal, arg) == NULL || !strlen (arg) ) {
		ER_throw (ER_DOMAIN);
	}
	
	// return value of argument at position
	return arg;
}*/

/* check if tag is boolean */
Boolean IsBool(EStackIndex i)
{
	return ESTACK (i) == FALSE_TAG || ESTACK (i) == TRUE_TAG;
}

/* check if tag is integer */
Boolean IsInt(EStackIndex i)
{
	return ESTACK (i) == NEGATIVE_INTEGER_TAG || ESTACK (i) == NONNEGATIVE_INTEGER_TAG;
}

/* check if tag is float */
Boolean IsFloat(EStackIndex i)
{
	return ESTACK (i) == FLOAT_TAG;
}

/* check if tag is complex float */
Boolean IsCFloat(EStackIndex i)
{
	Access_AMS_Global_Variables;
	
	Boolean b; 
	
	// is_complex_Float needs INTERNAL representation so
	// push_simplify_statements will not work in this case
	push_internal_simplify (i);
	
	b = is_complex_Float (top_estack); 
	
	delete_expression (top_estack);
	
	return b;
}

/* check if tag is string */
Boolean IsString(EStackIndex i)
{
	return ESTACK (i) == STR_DATA_TAG;
}

/* check if tag is true */
Boolean IsTrue(EStackIndex i)
{
	return ESTACK (i) == TRUE_TAG;
}

/* all members of list or mat have type 
also works for scalar expressions */
Boolean IsTypeAggregate(EStackIndex argptr,
	Boolean (*funcPtr)(EStackIndex i))
{
	Access_AMS_Global_Variables;
	
	Boolean b = TRUE;
	
	push_simplify_statements (argptr);
	
	argptr = top_estack;
	
	// arg is matrix
	if (is_matrix (argptr)) {
		// first element of first row
		argptr -= 2;
		
		// loop over rows
		while (ESTACK (argptr) != END_TAG) {
			// row does not satisfy condition
			if (!all_tail (funcPtr, argptr)) {
				b = FALSE;
				break;
			}
			
			// first element of next row
			argptr = next_expression_index (argptr + 1);
			
			// begin of next row
			if (ESTACK (argptr) == LIST_TAG) {
				// first element of next row
				argptr--;
			}
		}
	// argument is list
	} else if (ESTACK (argptr) == LIST_TAG) {
		// first element of list
		argptr--;
		
		// check elements of list using function
		b = all_tail (funcPtr, argptr); 		
	} else {
		// check argument using function
		b = (*funcPtr) (argptr);	
	}		
	
	// delete simplified argument
	delete_expression (top_estack);
	          
	return b;
}

// S C A L A R S //////////////////////////////////////////////////////////////

/* simplify and read short */
signed short estack_to_short_simp (EStackIndex argptr)
{
    Access_AMS_Global_Variables;
	signed short s;
	
	// pushes approximated arg to top of stack
	NG_approxESI (argptr); 
	
	if (estack_to_short (top_estack, &s) != 1) {
		ER_throwVar (ER_DOMAIN);
	}
	
	// remove approximated arg from top of stack
	delete_expression (top_estack);	
	
	return s;
}

/* simplify and read unsigned short */
unsigned short estack_to_ushort_simp (EStackIndex argptr)
{
    Access_AMS_Global_Variables; 
	unsigned short s;
	
	// pushes the result of approximation to top of stack
	NG_approxESI (argptr); 
	
	if (estack_to_ushort (top_estack, &s) != 1) {
		ER_throwVar (ER_DOMAIN);
	}
	
	// delete approximated expression again
	delete_expression (top_estack);	
	
	return s;
}

/* get signed int from estack */
/*int GetSignIntArg(EStackIndex *ptr) // ok
{
	switch (ESTACK (*ptr)) { 
		case NEGATIVE_INTEGER_TAG: 
			return -GetIntArg (*ptr);  // ??? missing
			break;
		case NONNEGATIVE_INTEGER_TAG: 
			return GetIntArg (*ptr);	// ??? missing
			break;
		default:
			ER_throw (ER_DATATYPE);
			break;
	}
}*/

/* get signed int argument at position */
signed short GetSignShortArgPos(unsigned char pos)
{
	return estack_to_short_simp (EX_getArg (pos));   
}

/* get signed long from estack */
/*int GetSignLongArg(CESI *ptr)  // ok
{
	switch (GetArgType (*ptr)) { 
		case NEGINT_TAG: 
			return -GetLongLongArg (*ptr);
			break;
		case POSINT_TAG: 
			return GetLongLongArg (*ptr);
			break;
		default:
			ER_throw (ER_DATATYPE);
			break;
	}
}*/

/* get float from expression stack 
0/0, -1/0, 1/0 gives ER_DATATYPE */
double estack_number_to_Float_inf(EStackIndex ptr) // 
{   
    Access_AMS_Global_Variables;
	double f = 0;
	
	TRY	
		// does not push and leave anything on stack
		f = ForceFloat (ptr);
	ONERR
	    if (!is_free_of_tag (ptr, MINUS_INFINITY_TAG)) {
			f = FLOATTAB[FPI_NEGINF];	    
	    } else if (!is_free_of_tag (ptr, PLUS_INFINITY_TAG)) {
			f = FLOATTAB[FPI_POSINF];	    
	    } else if (!is_free_of_tag (ptr, UNDEFINED_TAG)) {
			f = FLOATTAB[FPI_INVALID];	    
	    } else {
	    	ER_throwVar (ER_DATATYPE);
	    }
	ENDTRY
	
	return f;
}

/* get int, float or complex from estack 
argument may be int, real or complex */
Complex estack_number_to_Complex(EStackIndex ptr)
{
	Access_AMS_Global_Variables; 

	EStackIndex i;
	Complex a;
	
	//push_simplify_statements (ptr);
	push_internal_simplify (ptr);
	i = top_estack;
	
	switch (ESTACK (i))
	{
		// argument is scalar
		case FLOAT_TAG:
		case NEGATIVE_INTEGER_TAG:
		case NONNEGATIVE_INTEGER_TAG:
		case MINUS_INFINITY_TAG:
		case PLUS_INFINITY_TAG:
		case UNDEFINED_TAG:
			a.real = estack_number_to_Float_inf (i);
			a.imag = 0;
			break;
		// argument is complex	
		case IM_RE_TAG:
			a = GetComplexFloatArg (&i);
			break;
		default:
			ER_throwVar (ER_DATATYPE);
	}

	delete_expression (top_estack);
	
	return a;
}

/* get complex float from estack
argument must be complex
??? handle polar format */
Complex GetComplexFloatArg(EStackIndex *ptr)
{
	Complex input;

	//  ... FLOAT_TAG ... FLOAT_TAG COMPLEX_TAG
	
	// check if expression is complex
	// (this is internal representation)
	if (ESTACK (*ptr) != IM_RE_TAG) {
		ER_throwVar (ER_DATATYPE);
	}	
	
	// skip the COMPLEX_TAG
	(*ptr)--;
	
	// get imaginary part
	// without - the sign gets inverted	
	input.imag = estack_number_to_Float_inf (*ptr);
	*ptr = next_expression_index (*ptr);
		
	// get real part
	input.real = estack_number_to_Float_inf (*ptr);
	*ptr = next_expression_index (*ptr);
		
	return input;
}

/* push boolean to stack */
void push_bool(Boolean b)
{
	push_quantum (b ? TRUE_TAG : FALSE_TAG);
}

/* push complex float to estack (16.7) 
must push in internal format - otherwise outer functions (for example abs ()
will give wrong results
??? handle polar format */
void push_complex_float(Complex input) // ok
{	
	Access_AMS_Global_Variables; 
	
	EStackIndex old_top;
	
	push_Float (input.real);
	old_top = top_estack;		
	push_Float (input.imag);		
	replace_top2_with_imre (old_top);
}

// S T R I N G S //////////////////////////////////////////////////////////////

/* get string argument from stack 
caution: returns only pointer to string on estack and not copy of the string 
so we will get errors if overwriting the string on the estack before using 
this pointer 
also works when var conatining string is passed insted of string literal */
const char *GetStrnArg(EStackIndex argptr)
{
	const char *s;
	
	// Argument must be string containing name of a variable
	if (ESTACK (argptr) != STR_DATA_TAG) {
		ER_throwVar (ER_ARG_MUST_BE_STRING);  
	}      
		
	// Get pointer to beginning of variable name
	s = (const char *)(next_expression_index(argptr - 1) + 2);   
	
	// string is empty
	if (!strlen (s)) {
		ER_throwVar (ER_DOMAIN);
	}
        
    return s;
}

/* get variable name from stack */ 
const char *GetVarArg(EStackIndex argptr)
{
	const char *s;
	
	if (ESTACK (argptr) != VAR_TAG) {
		ER_throwVar (EXPECTED_VAR_ERROR);  
	}      
		
	// Get pointer to beginning of variable name
	s = (const char *)(next_expression_index(argptr /*- 1*/) + 2);   
	
	// string is empty
	if (!strlen (s)) {
		ER_throwVar (ER_DOMAIN);
	}
    
    return s;
}

/* get string argument from estack and copy it to another variable
(to avoid loosing its content while overwriting the stack) */
void GetStrnArgCopy(EStackIndex ptr, 
	char *dest, 
	unsigned char destLen)
{
	Access_AMS_Global_Variables; 

	const char *str;
	
	// expand content of variable to string
	push_simplify_statements (ptr); 
	
	// get search string from estack
	str = GetStrnArg (top_estack);
		
	// string is to long
	if (strlen (str) > destLen) {
		// exit to avoid buffer overflow
		ER_throwVar (ER_DOMAIN);
	}
		
	// copy the string to a place independent from estack
	// (to avoid overwriting while pushing something on the stack)
	strcpy (dest, str);  
	
	// remove expanded string
	delete_expression (top_estack); 
}

/* get string argument at position 
??? should also be implemented for other get functions */
/*const char *GetStrnArgPos(short pos)
{
	CESI ptr = EX_getArg (pos);
	return GetStrnArg (ptr);	
}*/

/* push string to estack */
/*void push_AMS_string(CESI ptr)
{
	push_expr_quantum (ptr, STR_TAG);	
}*/

// L I S T S //////////////////////////////////////////////////////////////////

/* check if index is list but not matrix */
Boolean is_list(EStackIndex i)
{
	// false when i points to matrix
	return ESTACK (i) == LIST_TAG && ESTACK (i - 1) != LIST_TAG;
}

/* return type of list 
only the first cell is checked */
/*enum Tags ListType(CESI ptr)
{
	if (GetArgType (ptr) != LIST_TAG) {
		ER_throw (ER_ARGUMENT_MUST_BE_LIST);
	}

	return GetArgType (ptr - 1);	
}*/

/* count the number of elements in list */
/*unsigned short GetListDim(CESI ptr)
{	
	// number of list elements
	unsigned short i = 0;
	
	if (GetArgType (ptr) != LIST_TAG) {
		ER_throw (ER_ARGUMENT_MUST_BE_LIST);
	}
	
	// move index to first element of list
	// (without decrement we start at END_TAG and are not looping)
	--ptr;

	// while not at end of list
	while (GetArgType (ptr) != END_TAG) { 
		// count the element
		i++;
		
		// step to next list element
		ptr = next_expression_index (ptr);
	}

	return i;
}*/

/* get list of floats 
content is written to output (size of array is adjusted automatically)
returns number of elements */
unsigned short GetFloatListArg(EStackIndex ptr, 
	volatile HANDLE *output) // ok
{	
	Access_AMS_Global_Variables; 

	// pointer to simplified list
	EStackIndex topPtr;          
	
	// number of elements in list
	unsigned short i = 0;
	
	// array for elements of list
	double *array;
		
	// expand variable name to list if necessary
	push_simplify_statements (ptr);
    
    // do not manipulate top_estack directly
    topPtr = top_estack;
	
	// wrong argument type
	if (!is_list (topPtr)) {
		ER_throwVar (ER_ARGUMENT_MUST_BE_LIST);		
	}	
    	
	// move index to first element of list
	// (without decrement we start at END_TAG and are not looping)
	topPtr--;

	// just in case the caller has not initialized the handle
	// (otherwise we will get an address error when trying to reallocate an invalid handle)
	*output = H_NULL;

	// while not at end of list
	while (ESTACK (topPtr) != END_TAG) {
		// allocate space for additional list element
		*output = HeapReallocThrow(*output, (i + 1) * sizeof(double));
		
		array = HeapDeref (*output);

		// convert to float and copy to array
		// (in contrast with GetIntArg and GetFloatArg this
		// function does not change the pointer to point
		// to the next argument)
		array[i++] = estack_number_to_Float_inf (topPtr);
		
		// so me must advance the pointer to next element explicitly
		topPtr = next_expression_index (topPtr);
	}
		
	// delete simplified copy of list
	delete_expression (top_estack);
	
	// return length of list
	return i;
}

/* get list of expressions 
content is written to output (size of array is adjusted automatically)
returns number of elements */
unsigned short GetExpressionListArg(EStackIndex ptr,
	volatile HANDLE *output) // 
{	
	Access_AMS_Global_Variables; 

	// pointer to simplified list
	EStackIndex topPtr;          
	
	// number of elements in list
	unsigned short i = 0;
	
	// array for elements of list
	EStackIndex *array;
		
	// expand variable name to list if necessary
	push_simplify_statements (ptr);
    
    // do not manipulate top_estack directly
    topPtr = top_estack;
	
	// wrong argument type
	if (!is_list (topPtr)) {
		ER_throwVar (ER_ARGUMENT_MUST_BE_LIST);		
	}	
    	
	// move index to first element of list
	// (without decrement we start at END_TAG and are not looping)
	topPtr--;

	// just in case the caller has not initialized the handle
	// (otherwise we will get an address error when trying to reallocate an invalid handle)
	*output = H_NULL;

	// while not at end of list
	while (ESTACK (topPtr) != END_TAG) {
		// allocate space for additional list element
		*output = HeapReallocThrow(*output, (i + 1) * sizeof (EStackIndex));
		
		array = HeapDeref (*output);

		// store pointer to current list element in array
		array[i++] = topPtr;
		
		// so me must advance the pointer to next element explicitly
		topPtr = next_expression_index (topPtr);
	}
		
	// delete simplified copy of list
	delete_expression (top_estack);
	
	// return length of list
	return i;
}

/* push array of floats to estack creating list 
pushes NAN as undefined without requiriring special handling */
void push_float_list(double input[], 
	short length) // ok
{
	// end of list
	push_quantum (END_TAG);
	  
	// loop over float array
	while (--length >= 0) {
		push_Float (input[length]);
	}
  
	// beginning of list
	push_quantum (LIST_TAG);	
}

/* push array of expressions to estack creating list */
void push_expression_list(EStackIndex input[], 
	short length) // ok
{
	// end of list
	push_quantum (END_TAG);
	  
	// loop over float array
	while (--length >= 0) {
		push_expression (input[length]);
	}
  
	// beginning of list
	push_quantum (LIST_TAG);	
}

/* push array of floats to estack creating list in reverse order */
/*void push_float_reverse_list(float input[], 
	unsigned short length)
{
	unsigned short i;

	// end of list
	push_END_TAG ();
  
	// loop over float array
	for (i = 0; i < length; i++) {
		push_Float (input[i]);
	}
  
	// beginning of list
	push_LIST_TAG ();	
}*/

/* get list of complex floats 
content is written to output (size of array is adjusted automatically)
returns number of elements */
unsigned short GetComplexFloatListArg(EStackIndex ptr, 
	volatile HANDLE *output) // ok
{
	Access_AMS_Global_Variables; 
	
	EStackIndex topPtr;
	
	// number of elements in list
	unsigned short i = 0;
	
	Complex *array;
	
	push_simplify_statements (ptr);
	topPtr = top_estack; 
	
	// wrong argument type
	if (!is_list (topPtr)) {
		ER_throw (ER_ARGUMENT_MUST_BE_LIST);		
	}
	
	// move index to first element of list
	// (without decrement we start at END_TAG and are not looping)
	topPtr--;
	
	// just in case the caller has not initialized the handle
	// (otherwise we will get an address error when trying to reallocate an invalid handle)
	*output = H_NULL;

	// while not at end of list
	while (ESTACK (topPtr) != END_TAG) {
		// allocate space for additional list element
		*output = HeapReallocThrow(*output, (i + 1) * sizeof(Complex));

		array = HeapDeref (*output);

		// convert to float and copy to array
		// (this also advances the pointer to the next list element -
		// do not call next_expression_index)			
		array[i++] = estack_number_to_Complex (topPtr);
		
		topPtr = next_expression_index (topPtr);
	}
	
	delete_expression (top_estack);
		
	return i;	
}
 
/* push list of complex floats to estack */
void push_complex_float_list(Complex input[], 
	short length) // ok
{
	// end of list
	push_quantum (END_TAG);
  
	// loop over float array
	while (--length >= 0) {
		push_complex_float (input[length]);
	}
  
	// beginning of list
	push_quantum (LIST_TAG);		
}

/* push array of strings to estack creating list */
/*void push_string_list_handle(HANDLE input[], 
	short length) // ok
{
	// end of list
	push_END_TAG ();
  
	// loop over string pointer array
	while (--length >= 0) {
		push_ANSI_string ( HeapDeref (input[length]));
	}
  
	// beginning of list
	push_LIST_TAG ();	
}*/

/* push empty list */
/*void push_empty_list(void) // 
{
	// end of list
	push_END_TAG ();
    
	// beginning of list
	push_LIST_TAG ();	
}*/

/* push array of expressions to estack creating list */
/*void push_list(CESI input[], 
	short length)
{
	// end of list
	push_END_TAG ();
  
	// loop over float array
	while (--length >= 0) {
		push_expression (input[length]);
	}
  
	// beginning of list
	push_LIST_TAG ();	
}*/

/* reverse the order of the array elements 
??? additional arg for length */
/*void ReverseArray(CESI array[])
{
	// length of array
	unsigned short l = sizeof(array) / sizeof(CESI);
	
	// counting upward
	unsigned short i;
	
	// counting downward
	unsigned short j = l - 1;
	
	// temp storage for swapping
	CESI t;
	
	// loop over all elements
	for (i = 0; i < l; i++) {
		t = array[i]; // ??? use macro for swapping
		array[i] = array[j];
		array[j] = t;
		j--;
	}	
}*/

// M A T R I C E S ////////////////////////////////////////////////////////////

/* return type of matrix 
only the first cell is checked */
/*enum Tags MatrixType(CESI ptr)
{
	if (!IsMatrix (ptr)) {
		ER_throw (ER_ARG_MUST_BE_MATRIX);
	}
	
	return GetArgType (ptr - 2);
}*/

/* count the number of rows and cols in mat */
/*void GetMatDim(CESI ptr, 
	unsigned short *rows, 
	unsigned short *cols) // ???
{
	ESI argptr = (ESI)ptr;
	
	if (!is_matrix (argptr)) {
		ER_throw (ER_ARG_MUST_BE_MATRIX);				
	}
	
	// move index to first list=row of matrix
	(*argptr)--;
	
	*rows = 0;
	*cols = 0;

	// while not at end of matrix
	while (GetArgType (argptr) != END_TAG) { 
		// while not at end of row
		while (GetArgType (argptr) != END_TAG) { 
			// step to next list element
			argptr = next_expression_index (argptr);

			(*cols)++;
		}
		
		// step to next list element
		argptr = next_expression_index (argptr);

		(*rows)++;
	}

	*cols /= *rows;
}*/

/* get matrix of long ints
content is written to output (size of array is adjusted automatically)
also returns number of rows and colums
{ { ELEM... } { ELEM... } }
LIST_TAG LIST_TAG ELEM... END_TAG LIST_TAG ELEM... END_TAG END_TAG */
/*void GetLongintMatArg(CESI *argptr,
	HANDLE *output, 
	unsigned short *rows,
	unsigned short *cols) // ok
{
	HANDLE rowHdl = H_NULL;
	long *rowPtr;
	long *matPtr;
	unsigned short rowsize; 
	
	// argument is not matrix
	if (!is_matrix (*argptr)) {
		ER_throw (ER_ARG_MUST_BE_MATRIX);				
	}
		
	// move index to first list=row of matrix
	(*argptr)--;
	
	*rows = 0;
	*cols = 0;
	
	// while not at end of matrix
	while (GetArgType (*argptr) != END_TAG) {
		// get next row of matrix
		*cols = GetLongintListArg (argptr, &rowHdl);
		
		// memory size required to hold one row
		rowsize = (*cols) * sizeof (long);
		
		// allocate space for the next row
		*output = HeapReallocThrow(*output, ((*rows) + 1) * rowsize);
		
		// get pointers from handles
		rowPtr = HeapDeref (rowHdl);
		matPtr = HeapDeref (*output);
		
		// copy actual row to matrix with previous rows
		memcpy (matPtr + (*rows) * (*cols),	// destination
			rowPtr, 						// source
			rowsize);						// length	
			
		(*rows)++;	
	}
	
	// point to next argument instead of LIST_END_TAG
	// (otherwise getting next argument from estack will fail)
	(*argptr)--;
}*/

/* get matrix of floats 
content is written to output (size of array is adjusted automatically)
also returns number of rows and colums
{ { ELEM... } { ELEM... } }
END_TAG END_TAG ELEM... LIST_TAG END_TAG ELEM... LIST_TAG LIST_TAG */
void GetFloatMatArg(EStackIndex argptr, // 
	volatile HANDLE *output, 
	unsigned short *rows,
	unsigned short *cols)
{
    Access_AMS_Global_Variables;  

	EStackIndex topPtr;
	double *matPtr;
	unsigned short cells = 0;  
	
	// expand variable name to matrix if necessary
	push_simplify_statements (argptr);
    
    // do not manipulate top_estack directly
    topPtr = top_estack;

	// argument is not a matrix
	if (!is_matrix (topPtr)) {
		ER_throwVar (ER_ARG_MUST_BE_MATRIX);				
	}
		
	// skip the two leading list tags
	topPtr -= 2;
	
	// just in case the caller has not initialized the handle
	// (otherwise we will get an address error when trying to reallocate an invalid handle)
	*output = H_NULL;
	
	// initialize the dimensions of the matrix
	*rows = 0;
	*cols = 0;
	
	// while not at end of matrix
	while (ESTACK (topPtr) != END_TAG || ESTACK (topPtr - 1) != END_TAG) {
		// handle end of row
		if (ESTACK (topPtr) != END_TAG) {
			// allocate space for the next float
			*output = HeapReallocThrow(*output, (cells + 1) * sizeof (double));
			
			// get pointer from handle
			matPtr = HeapDeref (*output);
			
			matPtr[cells] = estack_number_to_Float_inf (topPtr); 
			
			// advance the pointer to next float in row
			topPtr = next_expression_index (topPtr);
	
			cells++;
			(*cols)++;	
		// handle cell of matrix
		} else {		
			// reset column index before reading next line
			*cols = 0;
			
			// set index of next line
			(*rows)++;
			
			// skip end and list tag
			topPtr -=2; 
		}
	}

	(*rows) = cells / (*cols);	

	// delete simplified matrix
	delete_expression (top_estack);	
}

/* get matrix of expressions 
content is written to output (size of array is adjusted automatically)
also returns number of rows and colums
{ { ELEM... } { ELEM... } }
END_TAG END_TAG ELEM... LIST_TAG END_TAG ELEM... LIST_TAG LIST_TAG */
void GetExpressionMatArg(EStackIndex argptr, // 
	volatile HANDLE *output, 
	unsigned short *rows,
	unsigned short *cols)
{
    Access_AMS_Global_Variables;  

	EStackIndex topPtr;
	EStackIndex *matPtr;
	unsigned short cells = 0;  
	
	// expand variable name to matrix if necessary
	push_simplify_statements (argptr);
    
    // do not manipulate top_estack directly
    topPtr = top_estack;

	// argument is not a matrix
	if (!is_matrix (topPtr)) {
		ER_throwVar (ER_ARG_MUST_BE_MATRIX);				
	}
		
	// skip the two leading list tags
	topPtr -= 2;
	
	// just in case the caller has not initialized the handle
	// (otherwise we will get an address error when trying to reallocate an invalid handle)
	*output = H_NULL;
	
	// initialize the dimensions of the matrix
	*rows = 0;
	*cols = 0;
	
	// while not at end of matrix
	while (ESTACK (topPtr) != END_TAG || ESTACK (topPtr - 1) != END_TAG) {
		// handle end of row
		if (ESTACK (topPtr) != END_TAG) {
			// allocate space for the next float
			*output = HeapReallocThrow(*output, (cells + 1) * sizeof (EStackIndex));
			
			// get pointer from handle
			matPtr = HeapDeref (*output);
			
			matPtr[cells] = topPtr; 
			
			// advance the pointer to next float in row
			topPtr = next_expression_index (topPtr);
	
			cells++;
			(*cols)++;	
		// handle cell of matrix
		} else {		
			// reset column index before reading next line
			*cols = 0;
			
			// set index of next line
			(*rows)++;
			
			// skip end and list tag
			topPtr -=2; 
		}
	}

	(*rows) = cells / (*cols);	

	// delete simplified matrix
	delete_expression (top_estack);	
}

/* push matrix of long ints to estack */
/*void push_longint_mat(long input[][], 
	short rows, 
	unsigned short cols) // ok
{
  	// end of matrix
	push_END_TAG ();
  
	// loop over rows
 	while (--rows >= 0) {	
		push_longint_list ((long *)input + rows * cols, cols); 
	}
  
	// beginning of matrix
	push_LIST_TAG ();	
}*/

/* push matrix of floats to estack
array (one-dimensional using explicit pointer arithmetic)
number of rows
number of columns */
void push_float_mat(double *input, 
	short rows, 
	unsigned short cols) // ok
{
  	// end of matrix
	push_quantum (END_TAG);
  
	// loop over rows
 	while (--rows >= 0) {	
		push_float_list ((double *)input + rows * cols, cols); 
	}
  
	// beginning of matrix
	push_quantum (LIST_TAG);	
}

/* push matrix of expressions to estack
array (one-dimensional using explicit pointer arithmetic)
number of rows
number of columns */
void push_expression_mat(EStackIndex *input, 
	short rows, 
	unsigned short cols) // ok
{
  	// end of matrix
	push_quantum (END_TAG);
  
	// loop over rows
 	while (--rows >= 0) {	
		push_expression_list ((EStackIndex *)input + rows * cols, cols); 
	}
  
	// beginning of matrix
	push_quantum (LIST_TAG);	
}

/* get matrix of complex floats 
content is written to output (size of array is adjusted automatically)
also returns number of rows and colums
{ { ELEM... } { ELEM... } }
LIST_TAG LIST_TAG ELEM... END_TAG LIST_TAG ELEM... END_TAG END_TAG */
void GetComplexFloatMatArg(EStackIndex argptr, // 
	volatile HANDLE *output, 
	unsigned short *rows,
	unsigned short *cols)
{
    Access_AMS_Global_Variables;  

	EStackIndex topPtr;
	Complex *matPtr;
	unsigned short cells = 0;  
	
	// expand variable name to matrix if necessary
	push_simplify_statements (argptr);
    
    // do not manipulate top_estack directly
    topPtr = top_estack;

	// argument is not a matrix
	if (!is_matrix (topPtr)) {
		ER_throwVar (ER_ARG_MUST_BE_MATRIX);				
	}
		
	// skip the two leading list tags
	topPtr -= 2;
	
	// just in case the caller has not initialized the handle
	// (otherwise we will get an address error when trying to reallocate an invalid handle)
	*output = H_NULL;

	// initialize the dimensions of the matrix
	*rows = 0;
	*cols = 0;
	
	// while not at end of matrix
	while (ESTACK (topPtr) != END_TAG || ESTACK (topPtr - 1) != END_TAG) {
		// handle end of row
		if (ESTACK (topPtr) != END_TAG) {
			// allocate space for the next float
			*output = HeapReallocThrow(*output, (cells + 1) * sizeof (Complex));
			
			// get pointer from handle
			matPtr = HeapDeref (*output);
			
			matPtr[cells] = estack_number_to_Complex (topPtr); 
			
			// advance the pointer to next float in row
			topPtr = next_expression_index (topPtr);
	
			cells++;
			(*cols)++;	
		// handle cell of matrix
		} else {		
			// reset column index before reading next line
			*cols = 0;
			
			// set index of next line
			(*rows)++;
			
			// skip end and list tag
			topPtr -=2; 
		}
	}

	(*rows) = cells / (*cols);	

	// delete simplified matrix
	delete_expression (top_estack);	
}

/* push matrix of complex floats to estack */
void push_complex_float_mat(Complex *input, 
	short rows, 
	unsigned short cols) // ok
{
  	// end of matrix
	push_quantum (END_TAG);
  
	// loop over rows
 	while (--rows >= 0) {	
		push_complex_float_list ((Complex *)input + rows * cols, cols); 
	}
  
	// beginning of matrix
	push_quantum (LIST_TAG);	
}
   
// U T I L I T I E S //////////////////////////////////////////////////////////

/* determine begin of stack */
/*void StackBegin(ESI *ptr) // 
{
	// iterate from top to bottom of stack
	// (first argument to last argument)
	while (GetArgType (*ptr) != END_TAG) {
		// get next argument
  		*ptr = next_expression_index (*ptr);		
	}
	
	(*ptr)--;
}*/
